@@ -18,37 +18,59 @@ from .utils import mc_sandwich
 
 
 class MCGLM(MCGLMMean, MCGLMVariance):
-
     __doc__ = """
-        MCGLM class that implements MCGLM algorithm. (Bonat, Jørgensen 2015)
+    MCGLM class that implements MCGLM stastical models. (Bonat, Jørgensen 2015)
         
-        It extends GLM for multi-responses and dependent components by fitting second-moment adjustment.
+    It extends GLM for multi-responses and dependent components by fitting second-moment assumptions.
 
-        Args:
-            endog : array_like
-                1d array of endogenous response variable. In case of multiple responses, the user must pass the responses on a list. Recommended method concat of Numpy library.  
-            exog : array_like
-                A dataset with the endogenous matrix in a Numpy fashion. Since the library doesn't set an intercept by default, the user must add it. In the case of multiple responses, the user must pass the design matrices as a python list. 
-            z : array_like
-                List with matrices components of the linear covariance matrix.
-            link : array_like, string or None
-                Specification for the link function. The MCGLM library implements the following options: identity, logit, power, log, probit, cauchy, cloglog, loglog, negativebinomial. In the case of None, the library chooses the identity link. In multiple responses, user must pass values as list.  
-            variance : array_like, string or None
-                Specification for the variance function. The MCGLM library implements the following options: constant, tweedie, binomialP, binomialPQ, power, geom_tweedie, poisson_tweedie. In the case of None, the library chooses the constant link. In multiple responses, user must pass values as list.   
-            offset : array_like or None
-                Offset for continuous or count. In multiple responses, user must pass values as list.   
-            Ntrial : array_like or None
-                Ntrial for binomial responses. In multiple responses, user must pass values as list.
-            power_fixed : array_like or None
-                Parameter that allows estimation of power when variance functions is either tweedie, geom_tweedie or poisson_tweedie. In multiple responses, user must pass values as list. 
-            maxiter : float or None
-                Number max of iterations. Defaults to 200.
-            tol : float or None
-                Threshold of minimum absolute change on paramaters. Defaults to 0.0001.
-            tuning : float or None
-                Step size parameter. Defaults to 0.5.
-            weights : array_like or None
-                Weight matrix. Defaults to None.
+    Parameters
+    ----------
+    endog : array_like
+        1d array of endogenous response variable. In case of multiple responses, the user must pass the responses on a list.  
+    exog : array_like
+        A dataset with the endogenous matrix in a Numpy fashion. Since the library doesn't set an intercept by default, the user must add it. In the case of multiple responses, the user must pass the design matrices as a python list. 
+    z : array_like
+        List with matrices components of the linear covariance matrix.
+    link : array_like, string or None
+        Specification for the link function. The MCGLM library implements the following options: identity, logit, power, log, probit, cauchy, cloglog, loglog, negativebinomial. In the case of None, the library chooses the identity link. In multiple responses, user must pass values as list.  
+    variance : array_like, string or None
+        Specification for the variance function. The MCGLM library implements the following options: constant, tweedie, binomialP, binomialPQ, geom_tweedie, poisson_tweedie. In the case of None, the library chooses the constant link. In multiple responses, user must pass values as list.   
+    offset : array_like or None
+        Offset for continuous or count. In multiple responses, user must pass values as list.   
+    Ntrial : array_like or None
+        Ntrial for binomial responses. In multiple responses, user must pass values as list.
+    power_fixed : array_like or None
+        Parameter that allows estimation of power when variance functions is either tweedie, geom_tweedie or poisson_tweedie. In multiple responses, user must pass values as list. 
+    maxiter : float or None
+        Number max of iterations. Defaults to 200.
+    tol : float or None
+        Threshold of minimum absolute change on paramaters. Defaults to 0.0001.
+    tuning : float or None
+        Step size parameter. Defaults to 0.5.
+    weights : array_like or None
+        Weight matrix. Defaults to None.
+        
+    Examples
+    ----------
+    >>> import statsmodels.api as sm
+    >>> data = sm.datasets.scotland.load()
+    >>> data.exog = sm.add_constant(data.exog)
+    
+    >>> model = sm.GLM(data.endog, data.exog, z=[mc_id(data.exog)],
+    ...                      link="log", variance="tweedie",
+    ...                      power=2, power_fixed=False)
+    
+    >>> model_results = model.fit()
+    >>> model_results.mu
+    >>> model_results.pearson_residuals
+    >>> model_results.aic
+    >>> model_results.bic
+    >>> model_results.loglikelihood
+    
+    Notes
+    -----
+    MCGLM is a brand new model, which provides a solid statistical model for fitting multi-responses non-gaussian, dependent, or independent data based on second-moment assumptions. When a user instantiates an mcglm object, she must specify attributes such as link, variance, and z matrices; it will drive the overall behavior of the model.
+    For more details, check articles and documentation provided.
     """
 
     def __init__(
@@ -64,7 +86,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
         power_fixed=None,
         maxiter=50,
         tol=0.001,
-        tuning=0.5,
+        tuning=1,
         weights=None,
     ):
         super(MCGLM, self).__init__()
@@ -112,15 +134,31 @@ class MCGLM(MCGLMMean, MCGLMVariance):
 
     @property
     def df_model(self):
-        return len(self._beta_initial[0]) - 1
+        """Calculates the degree of freedom for the model."""
+        return len(self._beta_initial[0]) + len(self._tau_initial[0])
 
     @property
     def df_resid(self):
-        return self._n_obs - len(self._beta_initial[0])
+        """Calculates the degree of freedom for the model residuals."""
+        return self._n_obs - self.df_model
 
     def __calculate_static_attributes(
         self, endog, exog, link, variance, z, offset, power, power_fixed, ntrial
     ):
+        """Base method to warm up the pivotal artifacts for model training. It fulfills the None with default values, sets the power value, and creates the base vectors.
+
+        Args:
+            endog (list or np.array): _description_
+            exog (list or np.array): _description_
+            link (list): _description_
+            variance (list): _description_
+            z (list): _description_
+            offset (list): _description_
+            power (list): _description_
+            power_fixed (list): _description_
+            ntrial (list): _description_
+        """
+
         def initial_power(variance):
             if variance in [
                 "binomialP",
@@ -202,6 +240,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
                 if set_power:
                     power_list.append(initial_power(variance[index]))
 
+            rho = list()
             for _ in range(int(n_targets * (n_targets - 1) / 2)):
                 rho.append(0)
 
@@ -271,7 +310,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             bic=bic,
             df_resid=self.df_resid,
             df_model=self.df_model,
-            mu=mu,
+            mu=mu.reshape(self._n_targets, -1),
             y_values=self._y_values,
             X=self._X,
             ntrial=self._ntrial,
@@ -287,16 +326,16 @@ class MCGLM(MCGLMMean, MCGLMVariance):
 
             size_residue = len(residue)
             dens = (
-                (-size_residue / 2) * np.log(2 * math.pi)
+                -1 * (size_residue / 2) * np.log(2 * math.pi)
                 - 0.5 * det_sigma
-                - 0.5 * (residue.T * inv_sigma * residue)
+                - (residue.T * inv_sigma * residue)
             )
             return dens.mean()
 
         residue = endog - mu
         det_sigma = slogdet(block_diag(c_values))[1]
         log_likelihood = gauss(
-            residue=residue, det_sigma=det_sigma, inv_sigma=block_diag(c_inverse)
+            residue=residue, det_sigma=det_sigma, inv_sigma=c_inverse
         )
 
         return log_likelihood
@@ -316,7 +355,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
         )
 
     def _fit(self):
-        """The _fit implements the inference pipeline for mcglm."""
+        """This method implements the core inference for MCGLM, by all the means of the two-moment assumptions."""
         W = (
             np.diag(np.ones(len(self._y_values)))
             if self._weights is None
@@ -341,6 +380,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
         dispersion_historical.append(dispersion)
         for iter in range(self._max_iter):
             # First moment
+
             (
                 new_regression,
                 quasi_score,
@@ -353,6 +393,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             mu_attributes, mu, mu_derivatives = self.calculate_mean_features(
                 self._link, new_regression, self._X, self._offset
             )
+
             (
                 new_dispersion,
                 c_inverse,
@@ -362,12 +403,12 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             ) = self.update_covariates(
                 mu_attributes, rho, power, tau, W, dispersion, mu
             )
+
             dispersion_historical.append(new_dispersion)
 
             regression, dispersion, rho, power, tau = self.__iteration_update(
                 new_regression, new_dispersion, rho, power, tau
             )
-
             if self.__check_stop_criterion(
                 regression_historical, dispersion_historical
             ):
@@ -379,7 +420,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             varcov,
             joint_inv_sensitivity,
             joint_variability,
-        ) = MCGLMParameters.parameters_attributes(
+        ) = MCGLMParameters._parameters_attributes(
             residue,
             mu_derivatives,
             W,
@@ -412,7 +453,9 @@ class MCGLM(MCGLMMean, MCGLMVariance):
         )
 
     def __get_dispersion_parameters(self, mu, rho, tau, dispersion):
+
         covariances = dispersion.tolist()
+
         if self._n_targets == 1:
             rho = np.array([rho])
         elif self._n_targets > 1:
@@ -427,9 +470,13 @@ class MCGLM(MCGLMMean, MCGLMVariance):
                 "mu": mu[index * self._n_obs : (index + 1) * self._n_obs],
             }
 
+            size_parameters = (
+                len(tau[index]) if self._power_fixed[index] else len(tau[index])
+            )
+
             scale_output = {
                 "scalelist": [
-                    round(covariances.pop(0), 3) for _ in range(len(tau[index]))
+                    round(covariances.pop(0), 3) for _ in range(size_parameters)
                 ]
             }
 
@@ -451,7 +498,10 @@ class MCGLM(MCGLMMean, MCGLMVariance):
     def __iteration_update(self, new_regression, new_dispersion, rho, power, tau):
 
         if self._n_targets == 1:
-            return new_regression, new_dispersion, rho, power, [new_dispersion]
+            if self._power_fixed[0]:
+                return new_regression, new_dispersion, rho, power, [new_dispersion]
+            else:
+                return new_regression, new_dispersion, rho, [new_dispersion[0]], [new_dispersion[1:].tolist()]
 
         if (isinstance(rho, list)) or (isinstance(rho, np.ndarray)):
             new_rho = new_dispersion[0 : len(rho)]
@@ -478,7 +528,7 @@ class MCGLM(MCGLMMean, MCGLMVariance):
                 additions += 1
             stack_index += additions
             new_tau.append(temp_tau)
-
+            
         return new_regression, new_dispersion, new_rho, new_power, new_tau
 
     def __create_dispersion_vector(self, rho, power, tau):
@@ -496,17 +546,49 @@ class MCGLM(MCGLMMean, MCGLMVariance):
         return covariance
 
     def __update_optimal_dispersion(self):
+        """Update optimal dispersion method calculates optimal values for regression parameters and the dispersion. It harnesses the GLM API of statsmodels for calculating those values."""
+
         def logit_est(endog, exog, offset):
+            """For the Logit link function, the method adjusts a GLM with Binomial family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(endog, exog, family=sm.families.Binomial(), offset=offset)
             mdl_results = mdl.fit()
             return mdl_results.params.values, mdl_results.scale
 
         def loglog_est(endog, exog, offset):
+            """For the LogLog link function, the method adjusts a GLM with Binomial family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(endog, exog, family=sm.families.Binomial(LogLog()), offset=offset)
             mdl_results = mdl.fit()
             return mdl_results.params.values, mdl_results.scale
 
         def cloglog_est(endog, exog, offset):
+            """For the CLogLog link function, the method adjusts a GLM with Binomial family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(
                 endog, exog, family=sm.families.Binomial(CLogLog()), offset=offset
             )
@@ -514,21 +596,61 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             return mdl_results.params.values, mdl_results.scale
 
         def cauchy_est(endog, exog, offset):
+            """For the Cauchy link function, the method adjusts a GLM with Binomial family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(endog, exog, family=sm.families.Binomial(cauchy()), offset=offset)
             mdl_results = mdl.fit()
             return mdl_results.params.values, mdl_results.scale
 
         def probit_est(endog, exog, offset):
+            """For the Cauchy link function, the method adjusts a GLM with Binomial family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(endog, exog, family=sm.families.Binomial(probit()), offset=offset)
             mdl_results = mdl.fit()
             return mdl_results.params.values, mdl_results.scale
 
         def identity_est(endog, exog, offset=None):
+            """For the Identity link function, the method adjusts a OLS. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = sm.OLS(endog, exog, offset=offset)
             mdl_results = mdl.fit()
             return mdl_results.params, mdl_results.scale
 
         def log_est(endog, exog, offset):
+            """For the Log link function, the method adjusts a GLM with Tweedie(power=1) family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(
                 endog, exog, family=sm.families.Tweedie(var_power=1), offset=offset
             )
@@ -536,6 +658,16 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             return mdl_results.params, mdl_results.scale
 
         def power_est(endog, exog, offset):
+            """For either the Power or Reciprocal link function, the method adjusts a GLM with Tweedie(power=2) family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(
                 endog, exog, family=sm.families.Tweedie(var_power=2), offset=offset
             )
@@ -543,6 +675,16 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             return mdl_results.params.values, mdl_results.scale
 
         def negative_binomial_est(endog, exog, offset):
+            """For the Negative Binomial link function, the method adjusts a GLM with Tweedie(power=2) family. It retrieves the parameters specified.
+
+            Args:
+                endog (array-like): outcome variable.
+                exog (array-like): independent variables.
+                offset (int): shift on response variable.
+
+            Returns:
+                tuple: regression parameters, dispersion parameter.
+            """
             mdl = GLM(
                 endog, exog, family=sm.families.Tweedie(var_power=2), offset=offset
             )
@@ -559,6 +701,8 @@ class MCGLM(MCGLMMean, MCGLMVariance):
             "cloglog": cloglog_est,
             "loglog": loglog_est,
             "negativebinomial": negative_binomial_est,
+            "inverse_power": log_est,
+            "reciprocal": power_est,
         }
 
         for target in range(self._n_targets):
@@ -576,53 +720,56 @@ class MCGLM(MCGLMMean, MCGLMVariance):
 
 
 class MCGLMParameters:
+    """According to MCGLM specification, grounded for frequentist inference traits, the estimation of resulting parameters converge asymptotically to a gaussian distribution with tuple mean-variance = (actual parameters, inverse of matrix Godambe). This property allows the calculation of pivotal traits regarding the parameters, such as: hypothesis testing and confidence interval.
+
+    This class implements every method related to this trait.
+    """
+
     @staticmethod
-    def parameters_attributes(
-        residue,
-        mu_derivative,
+    def _parameters_attributes(
+        resid,
+        mu_deriv,
         W,
         quasi_score,
-        mean_sensitivity,
-        mean_variability,
-        c_inverse,
-        c_values,
-        c_derivatives_componentes,
-        var_sensitivity,
+        mean_sens,
+        mean_varia,
+        c_inv,
+        c,
+        c_deriv,
+        var_sensi,
     ):
         """The parameters of MCGLM converge assymtoptically to a Normal Distribution. This trait allows some statistical inferences as hypothesis testing, confidence interval and so on. This method crafts three important matrices: Varcov: variance covariance matrix, joint_inv_sensitivity: inverse sensitivity, and joint_variability: the joint variability distribution."""
 
         variance_variability = MCGLMParameters.generate_var_variability(
-            residue, W, c_inverse, c_values, c_derivatives_componentes
+            resid, W, c_inv, c, c_deriv
         )
 
-        inv_cw = np.dot(c_inverse, W)
+        inv_cw = np.dot(c_inv, W)
 
-        s_cov_beta = MCGLMParameters.mc_cross_sensitivity(
-            cov_product=c_derivatives_componentes,
+        s_cov_beta = MCGLMParameters._mc_cross_sensitivity(
+            cov_product=c_deriv,
             columns_size=len(quasi_score),
         )
 
-        v_cov_beta = MCGLMParameters.mc_cross_variability(
-            cov_product=c_derivatives_componentes,
+        v_cov_beta = MCGLMParameters._mc_cross_variability(
+            cov_product=c_deriv,
             inv_cw=inv_cw,
-            res=residue,
-            d=mu_derivative,
+            res=resid,
+            d=mu_deriv,
         )
 
-        p1 = np.append(mean_variability, v_cov_beta.transpose(), axis=0)
+        p1 = np.append(mean_varia, v_cov_beta.transpose(), axis=0)
 
         p2 = np.append(v_cov_beta, variance_variability, axis=0)
         joint_variability = np.append(p1, p2, axis=1)
 
-        inv_J_beta = inv(mean_sensitivity)
+        inv_J_beta = inv(mean_sens)
         inv_S_beta = inv_J_beta
-        inv_S_cov = inv(var_sensitivity)
+        inv_S_cov = inv(var_sensi)
         mat0 = np.zeros((s_cov_beta.shape[1], s_cov_beta.shape[0]))
 
         cross_term = mc_sandwich(s_cov_beta, -inv_S_cov, inv_S_beta)
-
         p1 = np.append(inv_S_beta, cross_term, axis=0)
-
         p2 = np.append(mat0, inv_S_cov, axis=0)
 
         joint_inv_sensitivity = np.append(p1, p2, axis=1)
@@ -636,23 +783,21 @@ class MCGLMParameters:
             joint_variability,
         )
 
-    def mc_cross_sensitivity(cov_product, columns_size):
+    def _mc_cross_sensitivity(cov_product, columns_size):
         nrow = len(cov_product)
         return np.zeros((nrow, columns_size))
 
-    def generate_var_variability(
-        residue, w, c_inverse, c_values, c_derivatives_componentes
-    ):
+    def generate_var_variability(res, w, c_inv, c_val, c_comp):
 
-        return MCGLMParameters.calculate_variability(
-            product=c_derivatives_componentes,
-            inv_C=c_inverse,
-            C=c_values,
-            res=residue,
+        return MCGLMParameters._calculate_variability(
+            product=c_comp,
+            inv_C=c_inv,
+            C=c_val,
+            res=res,
             W=w,
         )
 
-    def calculate_variability(product, inv_C, C, res, W):
+    def _calculate_variability(product, inv_C, C, res, W):
         n_par = len(product)
         we = [np.dot(product[index], inv_C) for index in range(n_par)]
 
@@ -660,11 +805,11 @@ class MCGLMParameters:
         sensitivity = MCGLMVariance.generate_sensitivity(product, W=W**2)
         W = np.diag(W).flatten()
 
-        variability = MCGLMParameters.mc_variability(sensitivity, we, k4, W)
+        variability = MCGLMParameters._mc_variability(sensitivity, we, k4, W)
 
         return variability
 
-    def mc_variability(sensitivity, we, k4, w):
+    def _mc_variability(sensitivity, we, k4, w):
         variability = np.array([])
         for position_row in range(len(we)):
             wi = np.diag(we[position_row])
@@ -680,13 +825,13 @@ class MCGLMParameters:
 
         return variability.reshape((len(we), len(we)))
 
-    def covprod(a, w, res):
+    def _covprod(a, w, res):
         calculation_sandwich = np.dot(np.dot(res, w), res)
         calculation_residue = np.dot(res.transpose(), a)
         product = np.dot(calculation_sandwich, calculation_residue)
         return product
 
-    def mc_cross_variability(cov_product, inv_cw, res, d):
+    def _mc_cross_variability(cov_product, inv_cw, res, d):
 
         wlist = [np.dot(cov, inv_cw) for cov in cov_product]
         a = np.dot(d.transpose(), inv_cw)
@@ -697,13 +842,19 @@ class MCGLMParameters:
         for cov in range(n_cov):
             for beta in range(n_beta):
                 cross_variability.append(
-                    MCGLMParameters.covprod(a[beta, :], wlist[cov], res)
+                    MCGLMParameters._covprod(a[beta, :], wlist[cov], res)
                 )
         cross_variability = np.array(cross_variability).reshape(n_cov, n_beta).T
         return cross_variability
 
 
 class MCGLMResults(GLMResults):
+    """MCGLM Class for generating and manipulating results of mcglm training. The main output goes by the method summary(), the classical statsmodels output. Therefore, the user can access the attributes "aic", "bic" e loglikelihood.
+
+    Args:
+        GLMResults: Class of statsmodels library for presenting results of GLM.
+    """
+
     def __init__(
         self,
         normalized_var_cov,
@@ -730,34 +881,47 @@ class MCGLMResults(GLMResults):
         X,
         ntrial,
     ):
-        self.normalized_var_cov = normalized_var_cov
-        self.nobs = nobs
-        self.n_targets = n_targets
-        self.y_names = y_names
-        self.regression = regression
-        self.dispersion = dispersion
-        self.n_iter = n_iter
-        self.residue = residue
-        self.rho = rho
-        self.tau = tau
-        self.power_list = power
-        self.link_list = link
-        self.variance_list = variance
-        self.power_fixed_list = power_fixed
-        self.p_log_likelihood = p_log_likelihood
-        self.p_aic = aic
-        self.p_bic = bic
-        self.df_resid = df_resid
-        self.df_model = df_model
-        self.mu_value = mu
-        self.y_values = y_values
-        self.X = X
-        self.ntrial = ntrial
+        self._normalized_var_cov = normalized_var_cov
+        self._nobs = nobs
+        self._n_targets = n_targets
+        self._y_names = y_names
+        self._regression = regression
+        self._dispersion = dispersion
+        self._n_iter = n_iter
+        self._residue = residue
+        self._rho = rho
+        self._tau = tau
+        self._power_list = power
+        self._link_list = link
+        self._variance_list = variance
+        self._power_fixed_list = power_fixed
+        self._p_log_likelihood = p_log_likelihood
+        self._p_aic = aic
+        self._p_bic = bic
+        self._df_resid = df_resid
+        self._df_model = df_model
+        self._mu_value = mu
+        self._y_values = y_values
+        self._X = X
+        self._ntrial = ntrial
         self.params = None
+        #self.bse = None
 
         self._use_t = False
         self.model = None
 
+    @property
+    def aic(self):
+        return self._p_aic
+
+    @property
+    def bic(self):
+        return self._p_bic
+
+    @property
+    def loglikelihood(self):
+        return self._p_log_likelihood
+
     @property
     def bse(self):
         """The standard errors of the parameter estimates."""
@@ -767,7 +931,7 @@ class MCGLMResults(GLMResults):
             bse_ = np.empty(len(self.params))
             bse_[:] = np.nan
         else:
-            bse_ = np.sqrt(np.diag(self.cov_params()))
+            bse_ = np.sqrt(np.diag(self.normalized_cov_params))
         return bse_
 
     @property
@@ -779,19 +943,19 @@ class MCGLMResults(GLMResults):
 
     @property
     def mu(self):
-        return self.mu_value
+        return self._mu_value
 
     @property
     def pvalues(self):
         """The two-tailed p values for the t-stats of the params."""
         if self.use_t:
-            df_resid = getattr(self, "df_resid_inference", self.df_resid)
+            df_resid = getattr(self, "df_resid_inference", self._df_resid)
             return stats.t.sf(np.abs(self.tvalues), df_resid) * 2
         else:
             return stats.norm.sf(np.abs(self.tvalues)) * 2
 
     @property
-    def pearson_residue(self):
+    def pearson_residuals(self):
         """
         Pearson residuals.  The Pearson residuals are defined as
         (`endog` - `mu`)/sqrt(VAR(`mu`)) where VAR is the distribution
@@ -799,20 +963,20 @@ class MCGLMResults(GLMResults):
         statsmodels.families.varfuncs for more information.
         """
         residuals = []
-        if self.n_targets == 1:
+        if self._n_targets == 1:
             mu = [self.mu]
         else:
-            mu = self.mu
+            mu = self.mu.reshape(self._n_targets, -1)
 
-        for index in range(self.n_targets):
+        for index in range(self._n_targets):
             residue = (
-                self.y_values[index * self.nobs : index * self.nobs + self.nobs]
+                self._y_values[index * self._nobs : index * self._nobs + self._nobs]
                 - mu[index]
             )
 
             variance = None
-            if self.variance_list[index] in ("binomialP", "binomialPQ"):
-                variance = self.variance_list[index]
+            if self._variance_list[index] in ("binomialP", "binomialPQ"):
+                variance = self._variance_list[index]
             else:
                 variance = "power"
 
@@ -821,8 +985,8 @@ class MCGLMResults(GLMResults):
                 ._generate_variance(
                     variance,
                     mu[index],
-                    self.power_list[index],
-                    self.ntrial[index],
+                    self._power_list[index],
+                    self._ntrial[index],
                 )
                 .get("variance_sqrt_output")
             )
@@ -842,7 +1006,7 @@ class MCGLMResults(GLMResults):
         total_anovas = list()
         for position_target in range(self.model.n_targets):
             dispersion = self.model.dispersion[position_target]["scalelist"][1:].copy()
-            dispersion_vcov = self.dispersion_vcov[position_target][1:, 1:].copy()
+            dispersion_vcov = self._dispersion_vcov[position_target][1:, 1:].copy()
 
             covariates_positions = [
                 list(j) for i, j in groupby(indexes_covariates[position_target])
@@ -899,14 +1063,14 @@ class MCGLMResults(GLMResults):
         ]
 
         top_right = [
-            ("No. Iterations:", [self.n_iter]),
-            ("No. Observations:", None),
-            ("Df Residuals:", None),
-            ("Df Model:", None),
+            ("No. Iterations:", [self._n_iter]),
+            ("No. Observations:", [self._nobs]),
+            ("Df Residuals:", [self._df_resid]),
+            ("Df Model:", [self._df_model]),
             ("Power-fixed:", [self.power_fixed]),
             ("pAIC", [self.paic]),
             ("pBIC", [self.pbic]),
-            ("pLogLik", [self.p_log_likelihood]),
+            ("pLogLik", [round(self._p_log_likelihood, 4)]),
         ]
 
         if hasattr(self, "cov_type"):
@@ -953,9 +1117,9 @@ class MCGLMResults(GLMResults):
         return summ
 
     def __add_rho_section(self, summ, alpha=0.05):
-        self.normalized_cov_params = self.rho_vcov
-        self.params = np.array(self.rho) if isinstance(self.rho, int) else self.rho
-        if isinstance(self.rho, (np.ndarray, np.generic)):
+        self.normalized_cov_params = self._rho_vcov
+        self.params = np.array(self._rho) if isinstance(self._rho, int) else self._rho
+        if isinstance(self._rho, (np.ndarray, np.generic)):
             summ.add_table_params(
                 self,
                 alpha=alpha,
@@ -977,25 +1141,25 @@ class MCGLMResults(GLMResults):
 
     def summary(self, yname=None, xname=None, title=None, alpha=0.05):
         """
-        Summarize the Regression Results
+        It generates the summary report as the sketch of classical "statsmodels" library. The summary shows all parameters found thoroughly, for each response.
         """
-        self.dispersion_vcov = []
+        self._dispersion_vcov = []
         self.betas_vcov = []
-        self.rho_vcov = []
+        self._rho_vcov = []
         self.power_vcov = []
 
         index_position = 0
-        cov_params = self.normalized_var_cov
+        cov_params = self._normalized_var_cov
 
-        if self.n_targets == 1:
-            self.rho_vcov = np.array([np.nan])
+        if self._n_targets == 1:
+            self._rho_vcov = np.array([np.nan])
             mu = [self.mu]
         else:
             mu = self.mu
 
-        for index in range(self.n_targets):
+        for index in range(self._n_targets):
 
-            n_betas = len(self.regression[index])
+            n_betas = len(self._regression[index])
             self.betas_vcov.append(
                 cov_params[
                     index_position : index_position + n_betas,
@@ -1004,17 +1168,17 @@ class MCGLMResults(GLMResults):
             )
             index_position += n_betas
 
-        if self.n_targets > 1:
-            self.rho_vcov = cov_params[
-                index_position : index_position + len(self.rho),
-                index_position : index_position + len(self.rho),
+        if self._n_targets > 1:
+            self._rho_vcov = cov_params[
+                index_position : index_position + len(self._rho),
+                index_position : index_position + len(self._rho),
             ]
 
-            index_position += len(self.rho)
+            index_position += len(self._rho)
 
-        for index in range(self.n_targets):
+        for index in range(self._n_targets):
 
-            if self.power_fixed_list[index]:
+            if self._power_fixed_list[index]:
                 self.power_vcov.append(np.array([np.nan]))
             else:
                 self.power_vcov.append(
@@ -1027,9 +1191,9 @@ class MCGLMResults(GLMResults):
                 )
                 index_position += 1
 
-            n_dispersion = len(self.dispersion[index]["scalelist"])
+            n_dispersion = len(self._dispersion[index]["scalelist"])
 
-            self.dispersion_vcov.append(
+            self._dispersion_vcov.append(
                 cov_params[
                     index_position : index_position + n_dispersion,
                     index_position : index_position + n_dispersion,
@@ -1042,27 +1206,28 @@ class MCGLMResults(GLMResults):
 
         smry = Summary()
 
-        for target_index in range(self.n_targets):
-            exog_names = self.X[target_index].columns.tolist()
-            y_name = self.y_names[target_index]
+        for target_index in range(self._n_targets):
+            exog_names = self._X[target_index].columns.tolist()
+            y_name = self._y_names[target_index]
 
-            self.link = self.link_list[target_index]
-            self.variance = self.variance_list[target_index]
-            self.power_fixed = self.power_fixed_list[target_index]
-            self.params = self.regression[target_index]
+            self.link = self._link_list[target_index]
+            self.variance = self._variance_list[target_index]
+            self.power_fixed = self._power_fixed_list[target_index]
+            self.params = self._regression[target_index]
             self.normalized_cov_params = self.betas_vcov[target_index]
 
-            self.loglikehood = self.p_log_likelihood
-            self.paic = self.p_aic
-            self.pbic = self.p_bic
+            self.loglikehood = self._p_log_likelihood
+            self.paic = self._p_aic
+            self.pbic = self._p_bic
 
             smry = self.__add_table_two_columns(smry, yname=y_name, xname=exog_names)
 
-            self.params = np.array(self.dispersion[target_index]["scalelist"])
-            self.normalized_cov_params = self.dispersion_vcov[target_index]
+            self.params = np.array(self._dispersion[target_index]["scalelist"])
+            self.normalized_cov_params = self._dispersion_vcov[target_index]
 
+            #self.bse = np.sqrt(np.diag(self.normalized_cov_params))
             smry = self.__add_dispersion(smry)
-            self.params = np.array([self.dispersion[target_index]["power"]])
+            self.params = np.array([self._dispersion[target_index]["power"]])
             self.normalized_cov_params = self.power_vcov[target_index]
 
             smry = self.__add_power(smry)
