@@ -18,27 +18,49 @@ from .utils import (
 
 class MCGLMCAttributes:
     """
-    The class "MCGLMCAttributes" has the task of calculating every C operations, used on throughout adjustments of mean and variance. This class has two interfaces, "c_inverse" and "c_complete"; one for each of two adjustment steps of MCGLM.
+    The class "MCGLMCAttributes" has the sake of calculating every C operations, used on throughout adjustments of mean and variance. This class has two interfaces, "c_inverse" and "c_complete"; one for each of two adjustment steps of MCGLM.
 
     The interface "c_inverse" crafts only inverse C, and the "c_complete" adds its derivatives and other features onto response. A Quasi-likelihood estimation needs only the inverse of "C" matrix. Therefore c_inverse saves computational resources by avoiding unnecessary operations on mean step adjustment.
     """
 
     def c_inverse(self, mu, power, rho, tau, full_response=False):
         """
-        A protected method to calculate inverse of C matrix, made specifically for quasi likelihood step. This method interacts with sigma and omega methods, passing the list of each parameter.
-
-        -----
-        params:
-
-        responds:
+        A method to generate only the inverse of the C matrix, explicitly made for the mean treatment step. This method interacts with sigma and omega amenities by list of each parameter.
 
+        Parameters
+        ----------
+            mu : array_like
+                A vetor with mean parameters.
+            power : float
+                Power parameter.
+            rho : float
+                Correlation parameter.
+            tau : float
+                Dispersion parameter.
+        Returns
+        -------
+            array_like or tuple : The inverse of C matrix and its components.
         """
         c_inverse = self.__generate_c_inverse(mu, power, rho, tau, full_response)
         return c_inverse
 
     def c_complete(self, mu, power, rho, tau):
         """
-        Full response
+        A method to generate the whole list of C components, explicitly made for the variance treatment step. This method interacts with sigma and omega crafting practices, passing the list of each parameter.
+
+        Parameters
+        ----------
+            mu : array_like
+                A vetor with mean parameters.
+            power : float
+                Power parameter.
+            rho : float
+                Correlation parameter.
+            tau : float
+                Dispersion parameter.
+        Returns
+        -------
+            tuple : A tuple with every component of C.
         """
         (
             diagonal_matrix,
@@ -53,11 +75,11 @@ class MCGLMCAttributes:
             c_inverse,
         ) = self.__generate_c_inverse(mu, power, rho, tau, full_response=True)
 
-        sigma_derivatives = self.__generate_c_derivatives(omega, mu, power)
+        sigma_derivatives = self.__generate_sigma_derivatives(omega, mu, power)
 
         core_matrix = np.kron(sigma_between, diagonal_matrix)
         sigma_chol_block_matrix_transpose = sigma_chol_block_matrix.transpose()
-        c_derivatives = self.__generate_derivative_cov(
+        c_derivatives = self.__generate_derivative_c(
             sigma_chol,
             sigma_chol_inv,
             sigma_derivatives,
@@ -80,6 +102,20 @@ class MCGLMCAttributes:
     def __generate_c_inverse(self, mu, power, rho, tau, full_response=False):
         """This method generates and retrieves the inverse of the C matrix, which is a pivotal component of the quasi-score calculation. Notwithstanding the mentioned, this method crafts many important artifacts throughout the time as omega, sigma, sigma Cholesky, the inverse of Cholesky sigma, sigma among responses, sigma block diagonal matrix alongside its inverse.
         Owing to the previously mentioned worthy artifacts crafted by the method, it can retrieve either only the c inverse matrix or all artifacts. The former is helpful for the method _c_inverse, whereas the former is for _c_complete.
+
+        Parameters
+        ----------
+            mu : array-like
+                A vetor with mean parameters.
+            power : float
+                Power parameter.
+            rho : float
+                Correlation parameter.
+            tau : float
+                Dispersion parameter.
+        Returns
+        -------
+            array_like : A matrix with the inverse matrix of C.
         """
         diagonal_matrix = diagonal(self._n_obs, np.ones(self._n_obs))
 
@@ -93,6 +129,7 @@ class MCGLMCAttributes:
             self._ntrial,
         )
         sigma_raw, sigma_chol, sigma_chol_inv = self.__parser_sigma(build_sigma)
+
         sigma_between, sigma_between_derivative = self._sigma_between_values(
             rho=rho, n_resp=self._n_targets
         )
@@ -122,8 +159,21 @@ class MCGLMCAttributes:
                 c_inverse,
             )
 
-    def __generate_c_derivatives(self, omega, mu, power):
+    def __generate_sigma_derivatives(self, omega, mu, power):
+        """Base method to generate derivatives related to sigma.
 
+        Parameters
+        ----------
+            omega : array-like
+                The omega matrix.
+            mu : array-like
+                A vetor with mean parameters.
+            power : float
+                The power parameter.
+        Returns
+        -------
+            _type_: _description_
+        """
         build_sigma = map(
             self._calculate_sigma_derivatives,
             mu,
@@ -138,7 +188,7 @@ class MCGLMCAttributes:
         sigma_derivative = self.__parser_sigma_derivatives(build_sigma)
         return sigma_derivative
 
-    def __generate_derivative_cov(
+    def __generate_derivative_c(
         self,
         sigma_chol,
         sigma_chol_inv,
@@ -150,7 +200,32 @@ class MCGLMCAttributes:
         diagonal_matrix,
         n_target,
     ):
+        """Base method to generate derivatives related to C matrix.
 
+        Parameters
+        ----------
+            sigma_chol : array_like
+                Sigma matrix decomposed by the Cholesky operation.
+            sigma_chol_inv : array_like
+                Inverse sigma matrix decomposed by the Cholesky operation.
+            sigma_derivatives : array_like
+                Derivatives related to the Sigma.
+            core_matrix : array_like
+                An intermediate matrix for Sigma operation
+            sigma_chol_block_matrix : array_like
+                A block diagonal matrix with all Sigmas.
+            sigma_chol_block_matrix_transpose : array_like
+                The transpose matrix of a block diagonal matrix with all Sigmas.
+            sigma_between_derivative : array_like
+                The deriviatives between sigmas.
+            diagonal_matrix : array_like
+                A diagonal matrix.
+            n_target : int
+                Total outcome variables.
+        Returns
+        -------
+            array_like : A matrix with derivatives related to the C Matrix.
+        """
         if n_target == 1:
 
             core_matrix_csr = sigma_between_derivative
@@ -229,6 +304,22 @@ class MCGLMCAttributes:
         sigma_chol_block_matrix_transpose,
         sigma_chol_block_matrix,
     ):
+        """base method to generate the C matrix.
+
+        Parameters
+        ----------
+            sigma_between : array_like
+                Matrix with inner values among sigmas.
+            diagonal_matrix : array_like
+                A diagonal matrix.
+            sigma_chol_block_matrix_transpose : array_like
+                The transpose matrix of a block diagonal matrix with all Sigmas.
+            sigma_chol_block_matrix : array_like
+                A block diagonal matrix with all Sigmas.
+        Returns
+        -------
+            array_like : A matrix with derivatives related to the C Matrix.
+        """
         kron_middle = np.kron(sigma_between, diagonal_matrix)
 
         c_computation = np.dot(
@@ -247,6 +338,23 @@ class MCGLMCAttributes:
         sigma_chol_block_matrix_transpose,
         core_matrix,
     ):
+        """Base method to calculate derivatives related to correlation parameters.
+
+        Parameters
+        ----------
+            d_sigmab : array_like
+                Matrix with inner values among sigmas.
+            sigma_chol_block_matrix : array_like
+                A block diagonal matrix with all Sigmas.
+            sigma_chol_block_matrix_transpose : array_like
+                The transpose matrix of a block diagonal matrix with all Sigmas.
+            core_matrix : array_like
+                A diagonal matrix.
+        Returns
+        -------
+            array_like : A matrix with derivatives related to the C Matrix.
+        """
+
         def multiplication_matrix(
             sigmab,
             sigma_chol_block_matrix,
@@ -269,8 +377,15 @@ class MCGLMCAttributes:
         ]
 
     def _generate_omega(self, tau):
-        """
-        It builds omega matrix for covariance link setted as identity.
+        """Base method to calculate derivatives related to correlation parameters.
+
+        Parameters
+        ----------
+            tau : list
+                List with dispersion parameters
+        Returns
+        -------
+            list : the results of matrix linear sum between tau and dependence matrices.
         """
         omega = []
         for target in range(self._n_targets):
@@ -280,17 +395,32 @@ class MCGLMCAttributes:
 
     def __generate_c_inverse_and_blocks(
         self,
-        sigma_chol: list,
-        sigma_chol_inv: list,
-        sigma_between: np.array,
+        sigma_chol,
+        sigma_chol_inv,
+        sigma_between,
         diagonal_matrix,
     ):
-        """ """
+        """Base method to calculate derivatives related to correlation parameters.
+
+        Parameters
+        ----------
+            sigma_chol : array_like
+                The sigma matrix decomposed by Cholesky.
+            sigma_chol_inv : array_like
+                The inverse of sigma matrix decomposed by Cholesky.
+            sigma_between : array_like
+                The inner-matrix among sigmas.
+            diagonal_matrix : array_like
+                A diagonal matrix.
+        Returns
+        -------
+            array_like : A matrix with derivatives related to the C Matrix.
+        """
         sigma_chol_block_matrix = block_diag(*sigma_chol)
         sigma_chol_inv_block_matrix = block_diag(*sigma_chol_inv)
 
         try:
-            ls_sigma_diagonal = np.linalg.inv(sigma_between)
+            ls_sigma_diagonal = inv(sigma_between)
         except Exception as e:
             ls_sigma_diagonal = np.array([[1]])
 
@@ -311,17 +441,32 @@ class MCGLMCAttributes:
     def _calculate_sigma(
         self, mu, power, omega, variance, Ntrial, covariance="identity"
     ):
-        """
-        Base method for computing variance-covariance matrix, based on variance function and omega matrix. This method will implement for cases where covariance is equal to identity, and variance falls in the list:
-        ['constant', 'tweedie', 'binomialP', 'binomialPQ', 'power', 'geom_tweedie', 'poisson_tweedie']
+        """Base method to calculate sigma
+
+        Parameters
+        ----------
+            mu : array_like
+                A vector with expected values.
+            power : float
+                A power parameter.
+            omega : array_like
+                The omega resulted matrix.
+            variance : str
+                The variance function.
+            Ntrial : int
+                The number of trial. Parameter for Binomial distribution.
+        Returns
+        -------
+            array_like : A matrix with Sigma values.
         """
         if isinstance(variance, list):
             variance = variance[0]
+
         if variance == "constant":
             sigma_raw = omega
             sigma_chol = cholesky(sigma_raw).T
             sigma_chol_inv = inv(sigma_chol)
-        elif variance in ["tweedie", "binomialP", "binomialPQ", "power"]:
+        elif variance in ["tweedie", "binomialP", "binomialPQ"]:
             if variance == "tweedie":
                 variance = "power"
 
@@ -359,6 +504,23 @@ class MCGLMCAttributes:
         """
         Base method for computing variance-covariance matrix, based on variance function and omega matrix. This method will implement for cases where covariance is equal to identity, and variance falls in the list:
         ['constant', 'tweedie', 'binomialP', 'binomialPQ', 'power', 'geom_tweedie', 'poisson_tweedie']
+
+        Parameters
+        ----------
+            mu : array_like
+                A vector with expected values.
+            power : float
+                A power parameter.
+            variance : str
+                The variance function.
+            z : list
+                The list with z matrices for dependencies specification.
+            power_fixed : boolean
+                The specification of power estimation.
+            Ntrial : int
+                The number of trial. Parameter for Binomial distribution.
+            omegas : list
+                a list with omegas.
         """
         if isinstance(variance, list):
             variance = variance[0]
@@ -366,11 +528,11 @@ class MCGLMCAttributes:
 
         if variance == "constant":
             sigma_derivative = z
-        elif variance in ["tweedie", "binomialP", "binomialPQ", "power"]:
-            if variance == "tweedie":
-                variance = "power"
+        elif variance in ["tweedie", "binomialP", "binomialPQ"]:
+            variance_type = variance if variance != "tweedie" else "power"
+
             variance_components = self.__generate_variance(
-                variance_type=variance, mu=mu.get("mu"), power=power, Ntrial=Ntrial
+                variance_type=variance_type, mu=mu.get("mu"), power=power, Ntrial=Ntrial
             )
             sigma_derivative = [
                 mc_sandwich(
@@ -381,20 +543,16 @@ class MCGLMCAttributes:
                 for d_omega in z
             ]
             if not power_fixed:
-                if variance in ["power", "binomialP"]:
-                    sigma_derivative_power = np.array(
-                        [
-                            mc_sandwich_power(
-                                omega,
-                                variance_components.get("variance_sqrt_output"),
-                                variance_components.get(
-                                    "derivative_variance_sqrt_power"
-                                ),
-                            )
-                            for omega in omegas
-                        ]
+                if variance in ["tweedie", "binomialP"]:
+ 
+                    sigma_derivative_power = mc_sandwich_power(
+                        omegas,
+                        variance_components.get("variance_sqrt_output"),
+                        variance_components.get("derivative_variance_sqrt_power"),
                     )
-                    sigma_derivative = [sigma_derivative_power, *sigma_derivative]
+                    sigma_derivative.insert(0, sigma_derivative_power)
+                    #sigma_derivative.append(sigma_derivative_power)
+
                 elif variance == "binomialPQ":
                     sigma_derivative_p = mc_sandwich(
                         omegas,
@@ -429,7 +587,7 @@ class MCGLMCAttributes:
                     variance_components.get("variance_sqrt_output"),
                     variance_components.get("derivative_variance_sqrt_power"),
                 )
-                sigma_derivative = [sigma_derivative_power, *sigma_derivative]
+                sigma_derivative.insert(0, sigma_derivative_power)
         return dict(sigma_derivative=sigma_derivative)
 
     def _sigma_between_values(self, rho, n_resp):
@@ -532,17 +690,31 @@ class MCGLMCAttributes:
     ):
         return self.__generate_variance(variance_type, mu, power, Ntrial)
 
-    def __generate_variance(
-        self, variance_type: str, mu: np.array, power: int = 1, Ntrial: list = 1
-    ):
-        if variance_type == "power":
-            return self.__power_variance_attributes(mu, power)
-        elif variance_type == "binomialP":
-            return self.__binomialp_variance_attributes(mu, power, Ntrial)
-        elif variance_type == "binomialPQ":
-            return self.__binomialpq_variance_attributes(mu, power, Ntrial)
+    def __generate_variance(self, variance_type, mu, power=1, Ntrial=1):
+        """Method to apply the variance function on the mean values.
 
-    def __power_variance_attributes(self, mu, power):
+        Parameters
+        ----------
+            variance_type : array_like
+                Type of variance
+            mu : array_like
+                A vector with expected values.
+            power : float
+                A power parameter.
+            Ntrial : int
+                The number of trial. Parameter for Binomial distribution.
+        Returns
+        -------
+            array_like : A matrix with Sigma values.
+        """
+        if variance_type == "power":
+            return self.__power_variance(mu, power)
+        elif variance_type == "binomialP":
+            return self.__binomialp_variance(mu, power, Ntrial)
+        elif variance_type == "binomialPQ":
+            return self.__binomialpq_variance(mu, power, Ntrial)
+
+    def __power_variance(self, mu, power):
         mu_power = mu**power
         sqrt_mu_power = np.sqrt(mu_power)
         n_len = len(mu)
@@ -559,7 +731,7 @@ class MCGLMCAttributes:
             derivative_variance_sqrt_mu=derivative_variance_sqrt_mu,
         )
 
-    def __binomialp_variance_attributes(self, mu, power, ntrial):
+    def __binomialp_variance(self, mu, power, ntrial):
         constant = 1 / ntrial
         mu_power = mu**power
         mu_power1 = (1 - mu) ** power
@@ -583,7 +755,7 @@ class MCGLMCAttributes:
             derivative_variance_sqrt_mu=derivative_variance_sqrt_mu,
         )
 
-    def __binomialpq_variance_attributes(self, mu, power, ntrial):
+    def __binomialpq_variance(self, mu, power, ntrial):
         constant = 1 / ntrial
         p = power[0]
         q = power[1]
